import { parse, visit, } from 'graphql';
import { default as picomatch } from 'picomatch';
import { getAutocompleteSuggestions, getDiagnostics, getHoverInformation, } from 'graphql-language-service-interface';
import { getVariablesJSONSchema, getOperationASTFacts, } from 'graphql-language-service-utils';
import { defaultSchemaLoader, } from './schemaLoader';
export class LanguageService {
    constructor({ parser, schemas, parseOptions, exteralFragmentDefinitions, }) {
        this._parser = parse;
        this._schemas = [];
        this._schemaCache = new Map();
        this._schemaLoader = defaultSchemaLoader;
        this._parseOptions = undefined;
        this._exteralFragmentDefinitionNodes = null;
        this._exteralFragmentDefinitionsString = null;
        this.getCompletion = (uri, documentText, position) => {
            const schema = this.getSchemaForFile(uri);
            if (!documentText || documentText.length < 1 || !schema?.schema) {
                return [];
            }
            return getAutocompleteSuggestions(schema.schema, documentText, position, undefined, this.getExternalFragmentDefinitions());
        };
        this.getDiagnostics = (uri, documentText, customRules) => {
            const schema = this.getSchemaForFile(uri);
            if (!documentText || documentText.length < 1 || !schema?.schema) {
                return [];
            }
            return getDiagnostics(documentText, schema.schema, customRules);
        };
        this.getHover = (uri, documentText, position) => {
            const schema = this.getSchemaForFile(uri);
            if (schema && documentText.length > 3) {
                return getHoverInformation(schema.schema, documentText, position);
            }
        };
        this.getVariablesJSONSchema = (uri, documentText) => {
            const schema = this.getSchemaForFile(uri);
            if (schema && documentText.length > 3) {
                try {
                    const documentAST = this.parse(documentText);
                    const operationFacts = getOperationASTFacts(documentAST, schema.schema);
                    if (operationFacts && operationFacts.variableToType) {
                        return getVariablesJSONSchema(operationFacts.variableToType);
                    }
                }
                catch (err) { }
            }
            return null;
        };
        this._schemaLoader = defaultSchemaLoader;
        if (schemas) {
            this._schemas = schemas;
        }
        if (parser) {
            this._parser = parser;
            this._cacheSchemas();
        }
        if (parseOptions) {
            this._parseOptions = parseOptions;
        }
        if (exteralFragmentDefinitions) {
            if (Array.isArray(exteralFragmentDefinitions)) {
                this._exteralFragmentDefinitionNodes = exteralFragmentDefinitions;
            }
            else {
                this._exteralFragmentDefinitionsString = exteralFragmentDefinitions;
            }
        }
    }
    _cacheSchemas() {
        this._schemas.forEach(schema => this._cacheSchema(schema));
    }
    _cacheSchema(schema) {
        const schemaResult = this._schemaLoader(schema, this.parse);
        return this._schemaCache.set(schema.uri, {
            ...schema,
            ...schemaResult,
        });
    }
    getSchemaForFile(uri) {
        const schema = this._schemas.find(schemaConfig => {
            if (!schemaConfig.fileMatch) {
                return false;
            }
            return schemaConfig.fileMatch.some(glob => {
                const isMatch = picomatch(glob);
                return isMatch(uri);
            });
        });
        if (schema) {
            const cacheEntry = this._schemaCache.get(schema.uri);
            if (cacheEntry) {
                return cacheEntry;
            }
            const cache = this._cacheSchema(schema);
            return cache.get(schema.uri);
        }
    }
    getExternalFragmentDefinitions() {
        if (!this._exteralFragmentDefinitionNodes &&
            this._exteralFragmentDefinitionsString) {
            const definitionNodes = [];
            try {
                visit(this._parser(this._exteralFragmentDefinitionsString), {
                    FragmentDefinition(node) {
                        definitionNodes.push(node);
                    },
                });
            }
            catch (err) {
                throw Error(`Failed parsing exteralFragmentDefinitions string:\n${this._exteralFragmentDefinitionsString}`);
            }
            this._exteralFragmentDefinitionNodes = definitionNodes;
        }
        return this._exteralFragmentDefinitionNodes;
    }
    async updateSchemas(schemas) {
        this._schemas = schemas;
        this._cacheSchemas();
    }
    updateSchema(schema) {
        const schemaIndex = this._schemas.findIndex(c => c.uri === schema.uri);
        if (schemaIndex < 0) {
            console.warn('updateSchema could not find a schema in your config by that URI', schema.uri);
            return;
        }
        this._schemas[schemaIndex] = schema;
        this._cacheSchema(schema);
    }
    addSchema(schema) {
        this._schemas.push(schema);
        this._cacheSchema(schema);
    }
    parse(text, options) {
        return this._parser(text, options || this._parseOptions);
    }
}
//# sourceMappingURL=LanguageService.js.map